@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -67,7 +68,7 @@ class CriticalSection {
   CriticalSection &operator=(const CriticalSection &);
 };
 
-grn_inline static int
+inline static int
 grn_dat_name(grn_ctx *ctx, grn_dat *dat, char *buffer, int buffer_size)
 {
   int name_size;
@@ -152,7 +153,7 @@ grn_dat_init(grn_ctx *ctx, grn_dat *dat)
   grn_table_modules_init(ctx, &(dat->token_filters));
   GRN_PTR_INIT(&(dat->token_filter_procs), GRN_OBJ_VECTOR, GRN_ID_NIL);
   CRITICAL_SECTION_INIT(dat->lock);
-  dat->is_dirty = GRN_FALSE;
+  dat->is_dirty = false;
 }
 
 static void
@@ -1763,7 +1764,7 @@ grn_dat_dirty(grn_ctx *ctx, grn_dat *dat)
     CriticalSection critical_section(&dat->lock);
     if (!dat->is_dirty) {
       uint32_t n_dirty_opens;
-      dat->is_dirty = GRN_TRUE;
+      dat->is_dirty = true;
       GRN_ATOMIC_ADD_EX(&(dat->header->n_dirty_opens), 1, n_dirty_opens);
       rc = grn_io_flush(ctx, dat->io);
     }
@@ -1795,7 +1796,7 @@ grn_dat_clean(grn_ctx *ctx, grn_dat *dat)
     CriticalSection critical_section(&dat->lock);
     if (dat->is_dirty) {
       uint32_t n_dirty_opens;
-      dat->is_dirty = GRN_FALSE;
+      dat->is_dirty = false;
       GRN_ATOMIC_ADD_EX(&(dat->header->n_dirty_opens), -1, n_dirty_opens);
       rc = grn_io_flush(ctx, dat->io);
     }
@@ -1815,7 +1816,7 @@ grn_dat_clear_dirty(grn_ctx *ctx, grn_dat *dat)
 
   {
     CriticalSection critical_section(&dat->lock);
-    dat->is_dirty = GRN_FALSE;
+    dat->is_dirty = false;
     dat->header->n_dirty_opens = 0;
     rc = grn_io_flush(ctx, dat->io);
   }
@@ -1823,22 +1824,22 @@ grn_dat_clear_dirty(grn_ctx *ctx, grn_dat *dat)
   return rc;
 }
 
-grn_bool
+bool
 grn_dat_is_corrupt(grn_ctx *ctx, grn_dat *dat)
 {
   if (!dat->io) {
-    return GRN_FALSE;
+    return false;
   }
 
   {
     CriticalSection critical_section(&dat->lock);
 
     if (grn_io_is_corrupt(ctx, dat->io)) {
-      return GRN_TRUE;
+      return true;
     }
 
     if (dat->header->file_id == 0) {
-      return GRN_FALSE;
+      return false;
     }
 
     char trie_path[PATH_MAX];
@@ -1849,11 +1850,11 @@ grn_dat_is_corrupt(grn_ctx *ctx, grn_dat *dat)
     if (::stat(trie_path, &stat) != 0) {
       SERR("[dat][corrupt] used path doesn't exist: <%s>",
            trie_path);
-      return GRN_TRUE;
+      return true;
     }
   }
 
-  return GRN_FALSE;
+  return false;
 }
 
 size_t
