@@ -382,6 +382,9 @@ command_object_inspect_column_data_value_compress(grn_ctx *ctx, grn_obj *column)
   case GRN_OBJ_COMPRESS_ZSTD:
     compress = "zstd";
     break;
+  case GRN_OBJ_COMPRESS_OPENZL:
+    compress = "openzl";
+    break;
   default:
     break;
   }
