@@ -125,12 +125,14 @@ grn_raw_string_substring(grn_ctx *ctx,
   grn_raw_string substring;
   substring.value = string->value + start;
   if (length < 0) {
-    substring.length = (size_t)((int64_t)(string->length - start) + length + 1);
+    int64_t substring_length = (int64_t)(string->length - start) + length + 1;
+    if (substring_length < 0) {
+      substring.length = 0;
+    } else {
+      substring.length = (size_t)substring_length;
+    }
   } else {
     substring.length = (size_t)length;
   }
-  if (substring.length < 0) {
-    substring.length = 0;
-  }
   return substring;
 }
