@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -315,7 +316,7 @@ grn_snip_cond_reinit(snip_cond *cond)
   cond->stopflag = SNIPCOND_NONSTOP;
 }
 
-grn_inline static char *
+static inline char *
 grn_snip_strndup(grn_ctx *ctx, const char *string, size_t string_len)
 {
   char *copied_string;
@@ -329,7 +330,7 @@ grn_snip_strndup(grn_ctx *ctx, const char *string, size_t string_len)
   return copied_string;
 }
 
-grn_inline static grn_rc
+static inline grn_rc
 grn_snip_cond_set_tag(grn_ctx *ctx,
                       const char **dest_tag,
                       size_t *dest_tag_len,
@@ -632,7 +633,7 @@ grn_snip_find_firstbyte(const char *string,
   return offset;
 }
 
-grn_inline static grn_rc
+static inline grn_rc
 grn_snip_set_default_tag(grn_ctx *ctx,
                          const char **dest_tag,
                          size_t *dest_tag_len,
