@@ -75,6 +75,8 @@ namespace {
   struct Options {
     /* Input */
     std::string model_name;
+    int32_t n_gpu_layers;
+    std::string centroid_column_name;
     uint32_t n_clusters;
     Quantizer quantizer;
     std::string code_column_name;
@@ -90,7 +92,10 @@ namespace {
     std::vector<grn_id> centroid_index_to_record_id;
     faiss::IndexFlatIP centroid_searcher;
 
-    /* For Quantization */
+    /* For clustering */
+    grn_obj *centroid_column;
+
+    /* For quantization */
     grn_obj *code_column;
 #ifdef GRN_FAISS_HAVE_RABITQ
     /* For RaBitQ */
@@ -103,6 +108,8 @@ namespace {
   options_init(grn_ctx *ctx, Options *options)
   {
     new (&options->model_name) std::string;
+    options->n_gpu_layers = GRN_LANGUAGE_MODEL_LOADER_N_GPU_LAYERS_DEFAULT;
+    new (&options->centroid_column_name) std::string;
     new (&options->code_column_name) std::string;
     new (&options->passage_prefix) std::string;
     new (&options->query_prefix) std::string;
@@ -120,6 +127,8 @@ namespace {
     new (&options->centroid_index_to_record_id) std::vector<grn_id>;
     new (&options->centroid_searcher) faiss::IndexFlatIP;
 
+    options->centroid_column = nullptr;
+
     options->code_column = nullptr;
 #ifdef GRN_FAISS_HAVE_RABITQ
     new (&options->random_rotation_matrix) faiss::RandomRotationMatrix;
@@ -139,10 +148,14 @@ namespace {
     options->model_name.~basic_string();
 
     // This may not work when this options is closed by grn_fin()
-    // because code_column may be alread closed.
+    // because centroid_column/code_column may be alread closed.
+    // if (options->centroid_column) {
+    //   grn_obj_unref(ctx, options->centroid_column);
+    // }
     // if (options->code_column) {
     //   grn_obj_unref(ctx, options->code_column);
     // }
+    options->centroid_column_name.~basic_string();
     options->code_column_name.~basic_string();
 
     options->passage_prefix.~basic_string();
@@ -167,6 +180,8 @@ namespace {
     grn_obj embedding;
     grn_obj transformed_embedding;
     grn_obj code;
+    // We need to keep token data until the next next() is called.
+    uint32_t current_centroid_id;
   };
 
   Tokenizer *
@@ -256,12 +271,20 @@ namespace {
 
       if (name_raw == "model") {
         set_string(options->model_name, raw_options, i);
+      } else if (name_raw == "n_gpu_layers") {
+        options->n_gpu_layers =
+          grn_vector_get_element_int32(ctx,
+                                       raw_options,
+                                       i,
+                                       options->n_gpu_layers);
       } else if (name_raw == "n_clusters") {
         options->n_clusters =
           grn_vector_get_element_uint32(ctx,
                                         raw_options,
                                         i,
                                         options->n_clusters);
+      } else if (name_raw == "centroid_column") {
+        set_string(options->centroid_column_name, raw_options, i);
       } else if (name_raw == "quantizer") {
         // TODO: RaBitQ and no quantization are only supported for now
       } else if (name_raw == "code_column") {
@@ -283,6 +306,81 @@ namespace {
       return nullptr;
     }
 
+    if (options->centroid_column_name.empty()) {
+      options->centroid_column = nullptr;
+    } else {
+      options->centroid_column =
+        grn_obj_column(ctx,
+                       data->lexicon,
+                       options->centroid_column_name.data(),
+                       options->centroid_column_name.size());
+      if (!options->centroid_column) {
+        char lexicon_name[GRN_TABLE_MAX_KEY_SIZE];
+        int lexicon_name_size =
+          grn_obj_name(ctx, data->lexicon, lexicon_name, sizeof(lexicon_name));
+        GRN_PLUGIN_ERROR(ctx,
+                         GRN_INVALID_ARGUMENT,
+                         "%s couldn't find centroid column: <%.*s.%s>",
+                         TOKENIZER_TAG,
+                         lexicon_name_size,
+                         lexicon_name,
+                         options->centroid_column_name.data());
+        close_options(ctx, options);
+        return nullptr;
+      }
+      grn_id range = grn_obj_get_range(ctx, options->centroid_column);
+      bool is_vector = grn_obj_is_vector_column(ctx, options->centroid_column);
+      if (range != GRN_DB_FLOAT32 || !is_vector) {
+        char lexicon_name[GRN_TABLE_MAX_KEY_SIZE];
+        int lexicon_name_size =
+          grn_obj_name(ctx, data->lexicon, lexicon_name, sizeof(lexicon_name));
+        const char *column_type = nullptr;
+        if (is_vector) {
+          column_type = "VECTOR";
+        } else if (grn_obj_is_index_column(ctx, options->centroid_column)) {
+          column_type = "INDEX";
+        } else {
+          column_type = "SCALAR";
+        }
+        GRN_PLUGIN_ERROR(ctx,
+                         GRN_INVALID_ARGUMENT,
+                         "%s centroid column must be Float32 vector: "
+                         "<%.*s.%s>: <%s>: <%s>",
+                         TOKENIZER_TAG,
+                         lexicon_name_size,
+                         lexicon_name,
+                         options->centroid_column_name.data(),
+                         grn_type_id_to_string_builtin(ctx, range),
+                         column_type);
+        close_options(ctx, options);
+        return nullptr;
+      }
+      if (data->lexicon->header.domain != GRN_DB_UINT32) {
+        char lexicon_name[GRN_TABLE_MAX_KEY_SIZE];
+        int lexicon_name_size =
+          grn_obj_name(ctx, data->lexicon, lexicon_name, sizeof(lexicon_name));
+        char lexicon_type_name[GRN_TABLE_MAX_KEY_SIZE];
+        int lexicon_type_name_size =
+          grn_table_get_key(ctx,
+                            grn_ctx_db(ctx),
+                            data->lexicon->header.domain,
+                            lexicon_type_name,
+                            sizeof(lexicon_type_name));
+        GRN_PLUGIN_ERROR(ctx,
+                         GRN_INVALID_ARGUMENT,
+                         "%s lexicon key type must be UInt32 "
+                         "with centroid column: <%.*s.%s>: <%.*s>",
+                         TOKENIZER_TAG,
+                         lexicon_name_size,
+                         lexicon_name,
+                         options->centroid_column_name.data(),
+                         lexicon_type_name_size,
+                         lexicon_type_name);
+        close_options(ctx, options);
+        return nullptr;
+      }
+    }
+
     if (options->code_column_name.empty()) {
       GRN_PLUGIN_ERROR(ctx,
                        GRN_INVALID_ARGUMENT,
@@ -291,7 +389,6 @@ namespace {
       close_options(ctx, options);
       return nullptr;
     }
-
     options->code_column = grn_obj_column(ctx,
                                           data->source_table,
                                           options->code_column_name.data(),
@@ -299,7 +396,7 @@ namespace {
     if (!options->code_column) {
       GRN_PLUGIN_ERROR(ctx,
                        GRN_INVALID_ARGUMENT,
-                       "%s couldn't find code_column: <%s>",
+                       "%s couldn't find code column: <%s>",
                        TOKENIZER_TAG,
                        options->code_column_name.data());
       close_options(ctx, options);
@@ -311,6 +408,9 @@ namespace {
                                         loader,
                                         options->model_name.data(),
                                         options->model_name.size());
+    grn_language_model_loader_set_n_gpu_layers(ctx,
+                                               loader,
+                                               options->n_gpu_layers);
     options->model = grn_language_model_loader_load(ctx, loader);
     grn_language_model_loader_close(ctx, loader);
     if (!options->model) {
@@ -357,21 +457,49 @@ namespace {
       options->centroids.resize(options->n_dimensions * options->n_clusters);
       options->centroid_index_to_record_id.resize(options->n_clusters);
       size_t i = 0;
-      GRN_TABLE_EACH_BEGIN_FLAGS(ctx,
-                                 data->lexicon,
-                                 cursor,
-                                 id,
-                                 GRN_CURSOR_BY_ID)
-      {
-        void *key;
-        auto key_size = grn_table_cursor_get_key(ctx, cursor, &key);
-        memcpy(options->centroids.data() + (options->n_dimensions * i),
-               key,
-               key_size);
-        options->centroid_index_to_record_id[i] = id;
-        ++i;
+      if (options->centroid_column) {
+        grn_obj centroid;
+        GRN_FLOAT32_INIT(&centroid, GRN_OBJ_VECTOR | GRN_OBJ_DO_SHALLOW_COPY);
+        GRN_TABLE_EACH_BEGIN_FLAGS(ctx,
+                                   data->lexicon,
+                                   cursor,
+                                   id,
+                                   GRN_CURSOR_BY_ID)
+        {
+          // This is for avoiding needless copy but this is tricky...
+          GRN_TEXT_SET_REF(&centroid,
+                           options->centroids.data() +
+                             (options->n_dimensions * i),
+                           0);
+          // The above 0 length and this sets rest
+          // size. grn_obj_get_value() copies a column value without
+          // changing GRN_BULK_HEAD(&centroid) if &centroid has enough
+          // rest size.
+          centroid.u.b.tail =
+            centroid.u.b.head + (sizeof(float) * options->n_dimensions);
+          grn_obj_get_value(ctx, options->centroid_column, id, &centroid);
+          options->centroid_index_to_record_id[i] = id;
+          ++i;
+        }
+        GRN_TABLE_EACH_END(ctx, cursor);
+        GRN_OBJ_FIN(ctx, &centroid);
+      } else {
+        GRN_TABLE_EACH_BEGIN_FLAGS(ctx,
+                                   data->lexicon,
+                                   cursor,
+                                   id,
+                                   GRN_CURSOR_BY_ID)
+        {
+          void *key;
+          auto key_size = grn_table_cursor_get_key(ctx, cursor, &key);
+          memcpy(options->centroids.data() + (options->n_dimensions * i),
+                 key,
+                 key_size);
+          options->centroid_index_to_record_id[i] = id;
+          ++i;
+        }
+        GRN_TABLE_EACH_END(ctx, cursor);
       }
-      GRN_TABLE_EACH_END(ctx, cursor);
     }
 #ifdef GRN_FAISS_HAVE_MAYBE_OWNED_VECTOR
     options->centroid_searcher.codes =
@@ -495,6 +623,7 @@ namespace {
         }
       }
 
+      // Generate embeddings
       auto cursor = grn_table_cursor_open(ctx_,
                                           source_table_,
                                           nullptr,
@@ -521,8 +650,7 @@ namespace {
         return false;
       }
 
-      faiss::Clustering clustering(options_->n_dimensions,
-                                   options_->n_clusters);
+      grn_tokenizer_build_data_start_vectorize(ctx_, data_, n_source_records_);
       embeddings_path_ = grn_obj_path(ctx_, index_column_);
       embeddings_path_ += ".embeddings";
       size_t embeddings_size =
@@ -541,11 +669,43 @@ namespace {
         embeddings_raw_ + (embeddings_size / sizeof(float));
       GRN_BINARY_SET_REF(&embeddings_, embeddings_raw_, embeddings_size);
       GRN_BULK_REWIND(&embeddings_);
-      grn_language_model_inferencer_vectorize_in_batch(ctx_,
-                                                       options_->inferencer,
-                                                       cursor,
-                                                       source_column_,
-                                                       &embeddings_);
+      {
+        bool need_progress = grn_ctx_get_progress_callback(ctx_);
+        struct UserData {
+          grn_tokenizer_build_data *data;
+        } user_data;
+        user_data.data = data_;
+        auto progress_callback =
+          [](grn_ctx *ctx, grn_progress *progress, void *user_data) {
+            auto user_data_ = static_cast<UserData *>(user_data);
+            auto n_processed_records =
+              grn_progress_language_model_inferencer_get_n_processed_records(
+                ctx,
+                progress);
+            grn_tokenizer_build_data_processed_n_records(ctx,
+                                                         user_data_->data,
+                                                         n_processed_records);
+          };
+        if (need_progress) {
+          grn_language_model_inferencer_set_progress_callback(
+            ctx_,
+            options_->inferencer,
+            progress_callback,
+            &user_data);
+        }
+        grn_language_model_inferencer_vectorize_in_batch(ctx_,
+                                                         options_->inferencer,
+                                                         cursor,
+                                                         source_column_,
+                                                         &embeddings_);
+        if (need_progress) {
+          grn_language_model_inferencer_set_progress_callback(
+            ctx_,
+            options_->inferencer,
+            nullptr,
+            nullptr);
+        }
+      }
       grn_table_cursor_close(ctx_, cursor);
 
       if (GRN_BULK_VSIZE(&embeddings_) != embeddings_size) {
@@ -562,6 +722,10 @@ namespace {
         return false;
       }
 
+      // Compute clusters
+      grn_tokenizer_build_data_start_cluster(ctx_, data_, n_source_records_);
+      size_t n_centroids = options_->n_clusters;
+      faiss::Clustering clustering(options_->n_dimensions, n_centroids);
       if (options_->quantizer == Quantizer::RABITQ) {
 #ifdef GRN_FAISS_HAVE_RABITQ
         options_->random_rotation_matrix.apply_noalloc(
@@ -573,16 +737,36 @@ namespace {
       } else {
         clustering.train(n_source_records_, embeddings_raw_, index);
       }
-      size_t n_centroids = clustering.k;
       options_->centroid_index_to_record_id.reserve(n_centroids);
-      for (size_t i = 0; i < n_centroids; ++i) {
-        grn_id id = grn_table_add(ctx_,
-                                  lexicon_,
-                                  clustering.centroids.data() +
-                                    (options_->n_dimensions * i),
-                                  sizeof(float) * options_->n_dimensions,
-                                  nullptr);
-        options_->centroid_index_to_record_id.push_back(id);
+      if (options_->centroid_column) {
+        grn_obj centroid;
+        GRN_FLOAT32_INIT(&centroid, GRN_OBJ_VECTOR | GRN_OBJ_DO_SHALLOW_COPY);
+        for (size_t i = 0; i < n_centroids; ++i) {
+          uint32_t key = i;
+          grn_id id =
+            grn_table_add(ctx_, lexicon_, &key, sizeof(uint32_t), nullptr);
+          GRN_TEXT_SET_REF(&centroid,
+                           clustering.centroids.data() +
+                             (options_->n_dimensions * i),
+                           sizeof(float) * options_->n_dimensions);
+          grn_obj_set_value(ctx_,
+                            options_->centroid_column,
+                            id,
+                            &centroid,
+                            GRN_OBJ_SET);
+          options_->centroid_index_to_record_id.push_back(id);
+        }
+        GRN_OBJ_FIN(ctx_, &centroid);
+      } else {
+        for (size_t i = 0; i < n_centroids; ++i) {
+          grn_id id = grn_table_add(ctx_,
+                                    lexicon_,
+                                    clustering.centroids.data() +
+                                      (options_->n_dimensions * i),
+                                    sizeof(float) * options_->n_dimensions,
+                                    nullptr);
+          options_->centroid_index_to_record_id.push_back(id);
+        }
       }
       options_->centroids = std::move(clustering.centroids);
 #ifdef GRN_FAISS_HAVE_MAYBE_OWNED_VECTOR
@@ -599,6 +783,7 @@ namespace {
              options_->centroid_searcher.codes.size());
 #endif
       options_->centroid_searcher.ntotal = n_centroids;
+      grn_tokenizer_build_data_processed_n_records(ctx_, data_, n_centroids);
 
       return true;
     }
@@ -617,6 +802,7 @@ namespace {
         grn_bulk_space(ctx_, &code, options_->rabitq.code_size);
 #endif
       }
+      grn_tokenizer_build_data_start_load(ctx_, data_, n_source_records_);
       size_t i = 0;
       GRN_TABLE_EACH_BEGIN_FLAGS(ctx_,
                                  source_table_,
@@ -784,11 +970,22 @@ namespace {
                                                  indexes);
     const float *centroid = tokenizer->options->centroids.data() +
                             (tokenizer->options->n_dimensions * indexes[0]);
-    grn_token_set_data(ctx,
-                       token,
-                       reinterpret_cast<const char *>(centroid),
-                       sizeof(float) * tokenizer->options->n_dimensions);
-    grn_token_set_domain(ctx, token, GRN_DB_SHORT_BINARY);
+    if (tokenizer->options->centroid_column) {
+      // We need to keep token data until the next next() is
+      // called. If we use a local variable here, token data is
+      // invalid after this block.
+      tokenizer->current_centroid_id = indexes[0];
+      auto key =
+        reinterpret_cast<const char *>(&(tokenizer->current_centroid_id));
+      grn_token_set_data(ctx, token, key, sizeof(uint32_t));
+      grn_token_set_domain(ctx, token, GRN_DB_UINT32);
+    } else {
+      grn_token_set_data(ctx,
+                         token,
+                         reinterpret_cast<const char *>(centroid),
+                         sizeof(float) * tokenizer->options->n_dimensions);
+      grn_token_set_domain(ctx, token, GRN_DB_SHORT_BINARY);
+    }
     grn_token_set_status(ctx, token, GRN_TOKEN_LAST);
     if (grn_tokenizer_query_get_mode(ctx, query) == GRN_TOKENIZE_ADD) {
       if (tokenizer->options->quantizer == Quantizer::NONE) {
@@ -1074,7 +1271,28 @@ namespace {
     uint32_t n_probes = 10;
     uint32_t k = 10;
     if (n_args == 4 && args[3]->header.type == GRN_TABLE_HASH_KEY) {
-      // TODO: Parse options by grn_proc_options_parse()
+      int32_t arg_k = -1;
+      grn_rc rc = grn_proc_options_parse(ctx,
+                                         args[3],
+                                         tag,
+                                         "k",
+                                         GRN_PROC_OPTION_VALUE_INT32,
+                                         &arg_k,
+                                         NULL);
+      if (rc != GRN_SUCCESS) {
+        return ctx->rc;
+      }
+      if (arg_k == 0) {
+        return GRN_SUCCESS;
+      } else if (arg_k > 0) {
+        k = static_cast<uint32_t>(arg_k);
+      } else {
+        int32_t new_k = grn_table_size(ctx, table) + arg_k + 1;
+        if (new_k <= 0) {
+          return GRN_SUCCESS;
+        }
+        k = static_cast<uint32_t>(new_k);
+      }
     }
 
     if (GRN_TEXT_LEN(query) == 0) {
